searchNodes=[{"type":"module","title":"Membrane.H264.Parser","doc":"Membrane element providing parser for H264 encoded video stream. The parser: prepares and sends the appropriate stream format, based on information provided in the stream and via the element's options splits the incoming stream into h264 access units - each buffer being output is a Membrane.Buffer struct with a binary payload of a single access unit enriches the output buffers with the metadata describing the way the access unit is split into NAL units, type of each NAL unit making up the access unit and the information if the access unit hold a keyframe. The parser works in one of three possible modes, depending on the structure of the input buffers: :bytestream - each input buffer contains some part of h264 stream's payload, but not necessary a logical h264 unit (like NAL unit or an access unit). Can be used for i.e. for parsing the stream read from the file. :nalu_aligned - each input buffer contains a single NAL unit's payload :au_aligned - each input buffer contains a single access unit's payload The parser's mode is set automatically, based on the input stream format received by that element: Receiving %Membrane.RemoteStream{type: :bytestream} results in the parser mode being set to :bytestream Receiving %Membrane.H264.RemoteStream{alignment: :nalu} results in the parser mode being set to :nalu_aligned Receiving %Membrane.H264.RemoteStream{alignment: :au} results in the parser mode being set to :au_aligned The distinguishment between parser modes was introduced to eliminate the redundant operations and to provide a reliable way for rewriting of timestamps: in the :bytestream mode: if option :framerate is set to nil, the output buffers have their :pts and :dts set to nil if framerate is specified, :pts and :dts will be generated automatically, based on that framerate, starting from 0 This may only be used with h264 profiles :baseline and :constrained_baseline , where PTS==DTS . in the :nalu_aligned mode, the output buffers have their :pts and :dts set to :pts and :dts of the input buffer that was holding the first NAL unit making up given access unit (that is being sent inside that output buffer). in the :au_aligned mode, the output buffers have their :pts and :dts set to :pts and :dts of the input buffer (holding the whole access unit being output) Element options Passed via struct Membrane.H264.Parser.t/0 sps binary ( ) Default value: &quot;&quot; Sequence Parameter Set NAL unit binary payload - if absent in the stream, should be provided via this option. pps binary ( ) Default value: &quot;&quot; Picture Parameter Set NAL unit binary payload - if absent in the stream, should be provided via this option. framerate { pos_integer ( ) , pos_integer ( ) } | nil Default value: nil Framerate of the video, represented as a tuple consisting of a numerator and the denominator. Its value will be sent inside the output Membrane.H264 stream format. output_alignment :au | :nalu Default value: :au Alignment of the buffers produced as an output of the parser. If set to :au , each output buffer will be a single access unit. Otherwise, if set to :nalu , each output buffer will be a single NAL unit. Defaults to :au . skip_until_keyframe? boolean ( ) Default value: false Determines whether to drop the stream until the first key frame is received. Defaults to false. repeat_parameter_sets boolean ( ) Default value: false Repeat all parameter sets ( sps and pps ) on each IDR picture. Parameter sets may be retrieved from: The bytestream Parser options. Decoder Configuration Record, sent as decoder_configuration_record in Membrane.H264.RemoteStream stream format Pads :input Accepted formats: % RemoteStream { type : :bytestream } % H264 { alignment : alignment } when alignment in [ :nalu , :au ] % H264.RemoteStream { alignment : alignment } when alignment in [ :nalu , :au ] Direction: :input Availability: :always Flow control: :auto Demand unit: :buffers :output Accepted formats: % H264 { alignment : alignment , nalu_in_metadata? : true } when alignment in [ :nalu , :au ] Direction: :output Availability: :always Flow control: :auto Demand unit: nil","ref":"Membrane.H264.Parser.html"},{"type":"function","title":"Membrane.H264.Parser.options/0","doc":"Returns description of options available for this module","ref":"Membrane.H264.Parser.html#options/0"},{"type":"type","title":"Membrane.H264.Parser.t/0","doc":"Struct containing options for Membrane.H264.Parser","ref":"Membrane.H264.Parser.html#t:t/0"},{"type":"module","title":"Membrane.H264.Parser.AUSplitter","doc":"Module providing functionalities to divide the binary h264 stream into access units. The access unit splitter's behaviour is based on &quot;7.4.1.2.3 Order of NAL units and coded pictures and association to access units&quot; of the &quot;ITU-T Rec. H.264 (01/2012)&quot; specification. The most crucial part of the access unit splitter is the mechanism to detect new primary coded video picture. WARNING: Our implementation of that mechanism is based on: &quot;7.4.1.2.4 Detection of the first VCL NAL unit of a primary coded picture&quot; of the &quot;ITU-T Rec. H.264 (01/2012)&quot; , however it adds one more additional condition which, when satisfied, says that the given VCL NALu is a new primary coded picture. That condition is whether the picture is a keyframe or not.","ref":"Membrane.H264.Parser.AUSplitter.html"},{"type":"function","title":"Membrane.H264.Parser.AUSplitter.flush/1","doc":"Returns a list of NAL units which are hold in access unit splitter's state accumulator and sets that accumulator empty. These NAL units aren't proved to form a new access units and that is why they haven't yet been output by Membrane.H264.Parser.AUSplitter.split/2 .","ref":"Membrane.H264.Parser.AUSplitter.html#flush/1"},{"type":"function","title":"Membrane.H264.Parser.AUSplitter.new/0","doc":"Returns a structure holding a clear state of the access unit splitter.","ref":"Membrane.H264.Parser.AUSplitter.html#new/0"},{"type":"function","title":"Membrane.H264.Parser.AUSplitter.split/2","doc":"Splits the given list of NAL units into the access units. It can be used for a stream which is not completly available at the time of function invoction, as the function updates the state of the access unit splitter - the function can be invoked once more, with new NAL units and the updated state. Under the hood, split/2 defines a finite state machine with two states: :first and :second . The state :first describes the state before reaching the primary coded picture NALu of a given access unit. The state :second describes the state after processing the primary coded picture NALu of a given access unit.","ref":"Membrane.H264.Parser.AUSplitter.html#split/2"},{"type":"type","title":"Membrane.H264.Parser.AUSplitter.access_unit_t/0","doc":"A type representing an access unit - a list of logically associated NAL units.","ref":"Membrane.H264.Parser.AUSplitter.html#t:access_unit_t/0"},{"type":"opaque","title":"Membrane.H264.Parser.AUSplitter.t/0","doc":"A structure holding a state of the access unit splitter.","ref":"Membrane.H264.Parser.AUSplitter.html#t:t/0"},{"type":"module","title":"Membrane.H264.Parser.DecoderConfigurationRecord","doc":"Utility functions for parsing AVC Configuration Record. The structure of the record is described in section 5.2.4.1.1 of MPEG-4 part 15 (ISO/IEC 14496-15).","ref":"Membrane.H264.Parser.DecoderConfigurationRecord.html"},{"type":"function","title":"Membrane.H264.Parser.DecoderConfigurationRecord.parse/1","doc":"Parses the DCR.","ref":"Membrane.H264.Parser.DecoderConfigurationRecord.html#parse/1"},{"type":"type","title":"Membrane.H264.Parser.DecoderConfigurationRecord.t/0","doc":"Structure representing the Decoder Configuartion Record","ref":"Membrane.H264.Parser.DecoderConfigurationRecord.html#t:t/0"},{"type":"module","title":"Membrane.H264.Parser.Format","doc":"Module providing functionalities for preparing H264 format based on the parsed SPS NAL units.","ref":"Membrane.H264.Parser.Format.html"},{"type":"function","title":"Membrane.H264.Parser.Format.from_sps/2","doc":"Prepares the Membrane.H264.t() format based on the parsed SPS NALu. During the process, the function determines the profile of the h264 stream and the picture resolution.","ref":"Membrane.H264.Parser.Format.html#from_sps/2"},{"type":"module","title":"Membrane.H264.Parser.NALu","doc":"A module defining a struct representing a single NAL unit.","ref":"Membrane.H264.Parser.NALu.html"},{"type":"type","title":"Membrane.H264.Parser.NALu.t/0","doc":"A type defining the structure of a single NAL unit produced by the Membrane.H264.Parser.NALuParser . In the structure there ardqde following fields: parsed_fields - the map with keys being the NALu field names and the values being the value fetched from the NALu binary. They correspond to the NALu schemes defined in the section 7.3. of the &quot;ITU-T Rec. H.264 (01/2012)&quot; . prefix_length - number of bytes of the prefix used to split the NAL units in the bytestream. The prefix is defined as in: &quot;Annex B&quot; of the &quot;ITU-T Rec. H.264 (01/2012)&quot; . type - an atom representing the type of the NALu. Atom's name is based on the &quot;Table 7-1 – NAL unit type codes, syntax element categories, and NAL unit type classes&quot; of the &quot;ITU-T Rec. H.264 (01/2012)&quot; . payload - the binary, which parsing resulted in that structure being produced status - :valid , if the parsing was successfull, :error otherwise","ref":"Membrane.H264.Parser.NALu.html#t:t/0"},{"type":"module","title":"Membrane.H264.Parser.NALuParser","doc":"A module providing functionality of parsing a stream of binaries, out of which each is a payload of a single NAL unit.","ref":"Membrane.H264.Parser.NALuParser.html"},{"type":"function","title":"Membrane.H264.Parser.NALuParser.new/0","doc":"Returns a structure holding a clear NALu parser state.","ref":"Membrane.H264.Parser.NALuParser.html#new/0"},{"type":"function","title":"Membrane.H264.Parser.NALuParser.parse/2","doc":"Parses a binary representing a single NALu. Returns a structure that contains parsed fields fetched from that NALu. The input binary is expected to contain the prefix, defined as in the &quot;Annex B&quot; of the &quot;ITU-T Rec. H.264 (01/2012)&quot; .","ref":"Membrane.H264.Parser.NALuParser.html#parse/2"},{"type":"opaque","title":"Membrane.H264.Parser.NALuParser.t/0","doc":"A structure holding the state of the NALu parser.","ref":"Membrane.H264.Parser.NALuParser.html#t:t/0"},{"type":"module","title":"Membrane.H264.Parser.NALuSplitter","doc":"A module with functions responsible for splitting the h264 stream into the NAL units. The splitting is based on &quot;Annex B&quot; of the &quot;ITU-T Rec. H.264 (01/2012)&quot; .","ref":"Membrane.H264.Parser.NALuSplitter.html"},{"type":"function","title":"Membrane.H264.Parser.NALuSplitter.flush/1","doc":"Flushes the payload out of the splitter state. That function gets the payload from the inner state of the splitter and sets the payload in the inner state clean.","ref":"Membrane.H264.Parser.NALuSplitter.html#flush/1"},{"type":"function","title":"Membrane.H264.Parser.NALuSplitter.new/1","doc":"Returns a structure holding a NALu splitter state. By default, the inner unparsed_payload of the state is clean. However, there is a possibility to set that unparsed_payload to a given binary, provided as an argument of the new/1 function.","ref":"Membrane.H264.Parser.NALuSplitter.html#new/1"},{"type":"function","title":"Membrane.H264.Parser.NALuSplitter.split/2","doc":"Splits the binary into NALus sequence. Takes a binary h264 stream as an input and produces a list of binaries, where each binary is a complete NALu that can be passed to the Membrane.H264.Parser.NALuParser.parse/2 .","ref":"Membrane.H264.Parser.NALuSplitter.html#split/2"},{"type":"opaque","title":"Membrane.H264.Parser.NALuSplitter.t/0","doc":"A structure holding the state of the NALu splitter.","ref":"Membrane.H264.Parser.NALuSplitter.html#t:t/0"},{"type":"module","title":"Membrane.H264.Parser.NALuTypes","doc":"The module aggregating the mapping of from nal_unit_type fields of the NAL unit to the human-friendly name of a NALu type.","ref":"Membrane.H264.Parser.NALuTypes.html"},{"type":"function","title":"Membrane.H264.Parser.NALuTypes.get_type/1","doc":"The function which returns the human friendly name of a NALu type for a given nal_unit_type . The mapping is based on: &quot;Table 7-1 – NAL unit type codes, syntax element categories, and NAL unit type classes&quot; of the &quot;ITU-T Rec. H.264 (01/2012)&quot;","ref":"Membrane.H264.Parser.NALuTypes.html#get_type/1"},{"type":"type","title":"Membrane.H264.Parser.NALuTypes.nalu_type/0","doc":"A type representing all the possible human-friendly names of NAL unit types.","ref":"Membrane.H264.Parser.NALuTypes.html#t:nalu_type/0"},{"type":"extras","title":"Membrane H264 Plugin","doc":"Membrane H264 parser. It is the Membrane element responsible for parsing the incoming h264 stream. The parsing is done as a sequence of the following steps: splitting the h264 stream into stream NAL units, based on the &quot;Annex B&quot; of the &quot;ITU-T Rec. H.264 (01/2012)&quot; Parsing the NAL unit headers, so that to read the type of the NAL unit Parsing the NAL unit body with the appropriate scheme, based on the NAL unit type read in the step before Aggregating the NAL units into a stream of access units The output of the element is the incoming binary payload, enriched with the metadata describing the division of the payload into access units . It is part of Membrane Multimedia Framework .","ref":"readme.html"},{"type":"extras","title":"Membrane H264 Plugin - Installation","doc":"The package can be installed by adding membrane_h264_plugin to your list of dependencies in mix.exs : def deps do [ { :membrane_h264_plugin , &quot;~&gt; 0.5.0&quot; } ] end","ref":"readme.html#installation"},{"type":"extras","title":"Membrane H264 Plugin - Usage","doc":"The following pipeline takes H264 file, parses it, and then decodes it to the raw video. defmodule Decoding.Pipeline do use Membrane.Pipeline alias Membrane . { File , H264 } @impl true def handle_init ( _ctx , _opts ) do structure = child ( :source , % File.Source { location : &quot;test/fixtures/input-10-720p-main.h264&quot; } ) |&gt; child ( :parser , H264.Parser ) |&gt; child ( :decoder , H264.FFmpeg.Decoder ) |&gt; child ( :sink , % File.Sink { location : &quot;output.raw&quot; } ) { [ spec : structure ] , nil } end @impl true def handle_element_end_of_stream ( :sink , _ctx_ , state ) do { [ terminate : :normal ] , state } end end","ref":"readme.html#usage"},{"type":"extras","title":"Membrane H264 Plugin - Copyright and License","doc":"Copyright 2022, Software Mansion Licensed under the Apache License, Version 2.0","ref":"readme.html#copyright-and-license"},{"type":"extras","title":"LICENSE","doc":"Apache License Version 2.0, January 2004 http://www.apache.org/licenses/ TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION 1. Definitions. &quot;License&quot; shall mean the terms and conditions for use, reproduction, and distribution as defined by Sections 1 through 9 of this document. &quot;Licensor&quot; shall mean the copyright owner or entity authorized by the copyright owner that is granting the License. &quot;Legal Entity&quot; shall mean the union of the acting entity and all other entities that control, are controlled by, or are under common control with that entity. For the purposes of this definition, &quot;control&quot; means (i) the power, direct or indirect, to cause the direction or management of such entity, whether by contract or otherwise, or (ii) ownership of fifty percent (50%) or more of the outstanding shares, or (iii) beneficial ownership of such entity. &quot;You&quot; (or &quot;Your&quot;) shall mean an individual or Legal Entity exercising permissions granted by this License. &quot;Source&quot; form shall mean the preferred form for making modifications, including but not limited to software source code, documentation source, and configuration files. &quot;Object&quot; form shall mean any form resulting from mechanical transformation or translation of a Source form, including but not limited to compiled object code, generated documentation, and conversions to other media types. &quot;Work&quot; shall mean the work of authorship, whether in Source or Object form, made available under the License, as indicated by a copyright notice that is included in or attached to the work (an example is provided in the Appendix below). &quot;Derivative Works&quot; shall mean any work, whether in Source or Object form, that is based on (or derived from) the Work and for which the editorial revisions, annotations, elaborations, or other modifications represent, as a whole, an original work of authorship. For the purposes of this License, Derivative Works shall not include works that remain separable from, or merely link (or bind by name) to the interfaces of, the Work and Derivative Works thereof. &quot;Contribution&quot; shall mean any work of authorship, including the original version of the Work and any modifications or additions to that Work or Derivative Works thereof, that is intentionally submitted to Licensor for inclusion in the Work by the copyright owner or by an individual or Legal Entity authorized to submit on behalf of the copyright owner. For the purposes of this definition, &quot;submitted&quot; means any form of electronic, verbal, or written communication sent to the Licensor or its representatives, including but not limited to communication on electronic mailing lists, source code control systems, and issue tracking systems that are managed by, or on behalf of, the Licensor for the purpose of discussing and improving the Work, but excluding communication that is conspicuously marked or otherwise designated in writing by the copyright owner as &quot;Not a Contribution.&quot; &quot;Contributor&quot; shall mean Licensor and any individual or Legal Entity on behalf of whom a Contribution has been received by Licensor and subsequently incorporated within the Work. 2. Grant of Copyright License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable copyright license to reproduce, prepare Derivative Works of, publicly display, publicly perform, sublicense, and distribute the Work and such Derivative Works in Source or Object form. 3. Grant of Patent License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable (except as stated in this section) patent license to make, have made, use, offer to sell, sell, import, and otherwise transfer the Work, where such license applies only to those patent claims licensable by such Contributor that are necessarily infringed by their Contribution(s) alone or by combination of their Contribution(s) with the Work to which such Contribution(s) was submitted. If You institute patent litigation against any entity (including a cross-claim or counterclaim in a lawsuit) alleging that the Work or a Contribution incorporated within the Work constitutes direct or contributory patent infringement, then any patent licenses granted to You under this License for that Work shall terminate as of the date such litigation is filed. 4. Redistribution. You may reproduce and distribute copies of the Work or Derivative Works thereof in any medium, with or without modifications, and in Source or Object form, provided that You meet the following conditions: (a) You must give any other recipients of the Work or Derivative Works a copy of this License; and (b) You must cause any modified files to carry prominent notices stating that You changed the files; and (c) You must retain, in the Source form of any Derivative Works that You distribute, all copyright, patent, trademark, and attribution notices from the Source form of the Work, excluding those notices that do not pertain to any part of the Derivative Works; and (d) If the Work includes a &quot;NOTICE&quot; text file as part of its distribution, then any Derivative Works that You distribute must include a readable copy of the attribution notices contained within such NOTICE file, excluding those notices that do not pertain to any part of the Derivative Works, in at least one of the following places: within a NOTICE text file distributed as part of the Derivative Works; within the Source form or documentation, if provided along with the Derivative Works; or, within a display generated by the Derivative Works, if and wherever such third-party notices normally appear. The contents of the NOTICE file are for informational purposes only and do not modify the License. You may add Your own attribution notices within Derivative Works that You distribute, alongside or as an addendum to the NOTICE text from the Work, provided that such additional attribution notices cannot be construed as modifying the License. You may add Your own copyright statement to Your modifications and may provide additional or different license terms and conditions for use, reproduction, or distribution of Your modifications, or for any such Derivative Works as a whole, provided Your use, reproduction, and distribution of the Work otherwise complies with the conditions stated in this License. 5. Submission of Contributions. Unless You explicitly state otherwise, any Contribution intentionally submitted for inclusion in the Work by You to the Licensor shall be under the terms and conditions of this License, without any additional terms or conditions. Notwithstanding the above, nothing herein shall supersede or modify the terms of any separate license agreement you may have executed with Licensor regarding such Contributions. 6. Trademarks. This License does not grant permission to use the trade names, trademarks, service marks, or product names of the Licensor, except as required for reasonable and customary use in describing the origin of the Work and reproducing the content of the NOTICE file. 7. Disclaimer of Warranty. Unless required by applicable law or agreed to in writing, Licensor provides the Work (and each Contributor provides its Contributions) on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied, including, without limitation, any warranties or conditions of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are solely responsible for determining the appropriateness of using or redistributing the Work and assume any risks associated with Your exercise of permissions under this License. 8. Limitation of Liability. In no event and under no legal theory, whether in tort (including negligence), contract, or otherwise, unless required by applicable law (such as deliberate and grossly negligent acts) or agreed to in writing, shall any Contributor be liable to You for damages, including any direct, indirect, special, incidental, or consequential damages of any character arising as a result of this License or out of the use or inability to use the Work (including but not limited to damages for loss of goodwill, work stoppage, computer failure or malfunction, or any and all other commercial damages or losses), even if such Contributor has been advised of the possibility of such damages. 9. Accepting Warranty or Additional Liability. While redistributing the Work or Derivative Works thereof, You may choose to offer, and charge a fee for, acceptance of support, warranty, indemnity, or other liability obligations and/or rights consistent with this License. However, in accepting such obligations, You may act only on Your own behalf and on Your sole responsibility, not on behalf of any other Contributor, and only if You agree to indemnify, defend, and hold each Contributor harmless for any liability incurred by, or claims asserted against, such Contributor by reason of your accepting any such warranty or additional liability. END OF TERMS AND CONDITIONS APPENDIX: How to apply the Apache License to your work. To apply the Apache License to your work, attach the following boilerplate notice, with the fields enclosed by brackets &quot;[]&quot; replaced with your own identifying information. (Don't include the brackets!) The text should be enclosed in the appropriate comment syntax for the file format. We also recommend that a file or class name and description of purpose be included on the same &quot;printed page&quot; as the copyright notice for easier identification within third-party archives. Copyright 2022 Software Mansion Licensed under the Apache License, Version 2.0 (the &quot;License&quot;); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.","ref":"license.html"}]